;--------------------------------------------------
; Um die Ablaufautomatik erfolgreich zu integrieren, hier das Standardgerst fr den Einsatz
;--------------------------------------------------


;##################################################
; 1. Variablen deklarieren
; -> in den Variablendeklarationsteil
	
	;----------
	; Konstanten
	
	Const AA_BUTTON = 1
	Const AA_GAME_MESSAGE = 2
	Const AA_TRAIN = 3
	Const AA_PATH = 4
	Const AA_TRIGGER = 5
	Const AA_TIMER = 6
	
	; Codes fr Bedingungen
	Const AA_COND_Touch = 1
	Const AA_COND_Use = 2
	Const AA_COND_ReachTarget = 3
	Const AA_COND_ReachValue = 4
	
	; Codes fr Anweisungen
	Const AA_ORDER_Activate = 1
	Const AA_ORDER_Deactivate = 2
	Const AA_ORDER_Delete = 3
	Const AA_ORDER_SetMessage = 4
	Const AA_ORDER_SetTarget = 5
	
	
	Const AA_USED = 2

	;----------
	;Types
	
	; Entities sind die eigentlichen Objekte, die verwaltet werden
	Type AA_entity
		Field entity_type
		Field name$
		Field x%,y%,height%,width%
		Field state%				; Zustand (aktiv / inaktiv)
		Field visible%				; Entity ist sichtbar
		Field message$				; Text der angezeigt wird oder Nachricht, die mitgegeben wird
		Field output_type$			; Typ der Rckgabe, z.B. "Touch" oder "Use"
		Field m.AA_mobile			; bei beweglichen Entities
		Field u.AA_usable			; bei benutzbaren Entities
		Field c.AA_count
		;Field image, frame			; soll stattdessen ein Bild angezeigt werden
	End Type


	; Unterobjekt bei beweglichen Objekten
	; (Nicht jedes Objekt braucht z.B. eine Geschwindigkeitsvariable)
	Type AA_mobile
		Field xSpeed, ySpeed		; Geschwindigkeit auf den Achsen
		Field target$
		Field xTarget, yTarget		; Koordinaten des Ziels
	End Type


	; Unterobjekt bei benutzbaren Objekten
	Type AA_usable
	;	Field sound					; Sound, der bei Benutzung abgespielt wird
		Field deactivate			; Entity wird nach Benutzung deaktiviert
	End Type
	
	
	; Unterobjekt bei Zhl-Entities
	Type AA_count
		Field value
		Field max_value
		Field reset
	End Type


	; Outputs beeinflussen die Objekte untereinander, die Aktion bei einem Quell-Entity lst
	; durch den entsprechend aktivierten Output beim Ziel-Entity eine Aktion aus
	Type AA_output
		Field name$
		Field state%
	; wichtig fr das Quell-Entity
		Field source$
		Field condition$			; Bedingung fr Output, Auslseaktion
	
	; wichtig fr das Ziel-Entity
		Field order$				; Befehl, der ausgefhrt werden soll
		Field target$				; Output-Ziel
		Field message$				; kann ein neues Ziel sein etc.
		
		Field stay					; lscht Output nach Benutzung, wenn 0
		Field deactivate			; deaktiviert den Output nach Benutzung, lscht ihn aber nicht
	End Type



;##################################################
; 2. Bentigte Objekte und Outputs laden
; -> nach Variablendeklaration, vor Hauptschleife

; noch werden die Objekte durch Include eingebunden, spter knnten sie aus speziellen Dateien geladen werden
Include "entities.bb"
Include "outputs.bb"



;##################################################
; 3. Scriptsprache einsetzen
; -> in der Hauptschleife


;-------------------------------------------------------------------------------
; Aktion mit den Entities
For AA_e.AA_entity = Each AA_entity

	If AA_e\state = True
	
		Select AA_e\entity_type
		
			Case AA_TRIGGER
				
				If RectsOverlap(MouseX(),MouseY(),1,1,AA_e\x,AA_e\y,AA_e\width,AA_e\height)
				
					AA_e\output_type = AA_COND_Touch
					
				EndIf
		
				
				
			Case AA_GAME_MESSAGE
				
				Color 255,255,255	
				Text AA_e\x,AA_e\y,AA_e\message$
				
				
				
			Case AA_BUTTON
				
				If MouseDown(1)
				
					If RectsOverlap(MouseX(),MouseY(),1,1,AA_e\x,AA_e\y,AA_e\width,AA_e\height)
					
						AA_e\output_type = AA_COND_Use
						If AA_e\u\deactivate = True  AA_e\state = False
						
					EndIf
				
				EndIf
				
				
			Case AA_TRAIN
			
				Text AA_e\x+2,AA_e\y+20,AA_e\m\target$
				
					
					For f.AA_entity = Each AA_entity
					
						If f\name$ = AA_e\m\target$
						
							AA_e\m\xTarget = f\x
							AA_e\m\yTarget = f\y
							
							If (AA_e\x = AA_e\m\xTarget)
								If (AA_e\y = AA_e\m\yTarget)
								
									AA_e\m\target$ = f\message$
									
								EndIf
							EndIf
						
						EndIf
						
					Next
					
				; Bewegung, wenn Koordinaten nicht denen des Ziel-Entitys entsprechen
					If (AA_e\x < AA_e\m\xTarget)
						AA_e\x = AA_e\x + AA_e\m\xSpeed
					ElseIf (AA_e\x > AA_e\m\xTarget)
						AA_e\x = AA_e\x - AA_e\m\xSpeed
					EndIf
					If (AA_e\y < AA_e\m\yTarget)
						AA_e\y = AA_e\y + AA_e\m\ySpeed
					ElseIf (AA_e\y > AA_e\m\yTarget)
						AA_e\y = AA_e\y - AA_e\m\ySpeed
					EndIf
					
				; ansonsten Meldung
					If AA_e\x = AA_e\m\xTarget
						If AA_e\y = AA_e\m\yTarget
							
							AA_e\output_type = AA_COND_ReachTarget
							
						EndIf
					EndIf


			Case AA_TIMER
			
				; berprft die Startzeit mit der aktuellen Zeit und gibt Meldung, wenn
				; max_value berschritten wurde
				If MilliSecs() - AA_e\c\value > AA_e\c\max_value
				
					AA_e\output_type = AA_COND_ReachValue
					
					; ist reset wahr, wird der Zhler neu gesetzt
					If AA_e\c\reset = True  AA_e\c\value = MilliSecs()
					
				EndIf
				
				
			Case AA_COUNTER
			
				If AA_e\c\value = AA_e\c\max_value
				
					AA_e\output_type = AA_COND_ReachValue
					
					If AA_e\c\reset = True  AA_e\c\value = 0
				
				EndIf
				
				Text AA_e\x,AA_e\y+16,AA_e\c\value + "/" + AA_e\c\max_value
		
	End Select ;Select AA_e\entity_type
	
	EndIf

	;AA_outputs aktivieren
	For AA_out.AA_output = Each AA_output
	
		If AA_out\source$ = AA_e\name$
		
			If AA_out\condition$ = AA_e\output_type
			
				AA_out\state = True
				AA_e\output_type = 0
				
			EndIf
			
		EndIf
		
	Next
	
	;+++++++++++++++++++++++++++
	; Entities zeichnen
	; Sollte man sich entscheiden, den Type "AA_entity" um die Komponente "image" zu erweitern...
	Select AA_e\entity_type
	
		Case AA_TRIGGER  Color 127,127,0
		Case AA_GAME_MESSAGE  Color 255,255,255
		Case AA_BUTTON  Color 255,0,0
		Case AA_TRAIN  Color 127,127,127
		Case AA_PATH  Color 200,0,200
		Case AA_TIMER Color 0,100,100
		Case AA_COUNTER Color 127,0,127

	End Select

	; ... sieht diese Zeile so aus: "If e\visible = True  DrawImage e\image, e\x, e\y" [, e\frame]
	If AA_e\visible = True
		Rect AA_e\x, AA_e\y, AA_e\width, AA_e\height, 0
		;Text AA_e\x+2, AA_e\y+2, AA_e\state+")"+AA_e\name$
	EndIf
	;+++++++++++++++++++++++++++

Next; For AA_e.AA_entity = Each AA_entity	
	
;-------------------------------------------------------------------------------
; Realisierung der Outputs
For AA_out.AA_output = Each AA_output

	If AA_out\state = True
	
		For AA_e.AA_entity = Each AA_entity
		
			If AA_out\target$ = AA_e\name$; Ziel entspricht AA_entity-Name
			
				Select AA_out\order
				
					Case AA_ORDER_Activate
						AA_e\state = True
						If AA_e\entity_type = AA_TIMER  AA_e\c\value = MilliSecs()
					
					Case AA_ORDER_Deactivate
						AA_e\state = False
					
					Case AA_ORDER_Delete
						AA_e\state = AA_USED
						
					Case AA_ORDER_SetMessage
						AA_e\message$ = AA_out\message$
						
					Case AA_ORDER_SetTarget
						AA_e\m\target$ = AA_out\message$
						
					Case AA_ORDER_IncreaseValue
						AA_e\c\value = AA_e\c\value + 1
											
				End Select

			EndIf

			If AA_e\state = AA_USED  Delete AA_e.AA_entity

		Next
		
		If AA_out\deactivate = True  AA_out\state = False
		If AA_out\stay = False  AA_out\state = AA_USED
		
	EndIf
	
	If AA_out\state = AA_USED  Delete AA_out.AA_output

Next
;-------------------------------------------------------------------------------